package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.MesProductionRecordModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/20 21:20
 * @Modify:
 **/
@Service
public class MesProductionRecordService implements IMesProductionRecordService {

    @Autowired
    private MesTimeEfficientCfgRepository timeEfficientCfgRepository;

    @Autowired
    private MesProductionRecordRepository productionRecordRepository;

    @Autowired
    private MesProductionAssemblyRepository productionAssemblyRepository;

    @Autowired
    private MesCraftRepository mesCraftRepository;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;

    @Autowired
    private MesWorkCellRepository mesWorkCellRepository;

    @Override
    public List<MesProductionRecord> findProductionRecordList(String organizeCode, String productSn) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(productSn)) return null;
        return productionRecordRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.PRODUCT_SN},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), productSn});
    }

    @Override
    public List<MesProductionRecord> findProductionRecordList(String organizeCode, String productSn, String partNo) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(productSn) || StringUtils.isEmpty(partNo)) return null;
        return productionRecordRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.PRODUCT_SN, MesPcnExtConstWords.PART_NO},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), productSn, partNo});
    }

    @Override
    public List<MesProductionRecord> findProductionRecordList(String organizeCode, String productSn, String partNo, String processCode, String craftCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(productSn) || StringUtils.isEmpty(partNo) || StringUtils.isEmpty(processCode) || StringUtils.isEmpty(craftCode)) return null;
        return productionRecordRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.PRODUCT_SN, MesPcnExtConstWords.PART_NO, MesPcnExtConstWords.PROCESS_CODE, MesPcnExtConstWords.CRAFT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), productSn, partNo, processCode, craftCode});
    }

    @Override
    public List<MesProductionRecord> findProductionRecordList(String organizeCode, String productSn, String partNo, String craftCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(productSn) || StringUtils.isEmpty(partNo) || StringUtils.isEmpty(craftCode)) return null;
        return productionRecordRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.PRODUCT_SN, MesPcnExtConstWords.PART_NO, MesPcnExtConstWords.CRAFT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), productSn, partNo, craftCode});
    }

    @Override
    public List<MesProductionRecord> findProductionRecordList(String organizeCode, List<String> orderNoList, Integer reportStatus) {
        if(StringUtils.isEmpty(orderNoList) || CollectionUtils.isEmpty(orderNoList) || Objects.isNull(reportStatus)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if(orderNoList.size() == MesPcnExtConstWords.ONE) {
            DdlPreparedPack.getStringEqualPack(orderNoList.iterator().next(),MesPcnExtConstWords.WORK_ORDER_NO,packBean);
        } else{
            DdlPreparedPack.getInPackList(orderNoList,MesPcnExtConstWords.WORK_ORDER_NO,packBean);
        }
        DdlPreparedPack.getNumEqualPack(reportStatus,MesPcnExtConstWords.REPORT_TYPE,packBean);
        return productionRecordRepository.findByHqlWhere(packBean);
    }

    @Override
    public void updateProductionRecord(String organizeCode, String userName, String sn) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);

        DdlPreparedPack.getStringEqualPack(sn, "productSn", ddlPackBean);
        productionRecordRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime","reportStatus"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getValue() },ddlPackBean);
    }
    @Override
    public void updateProductionRecord(String organizeCode, String userName, Long id, String errorMsg) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(id, "id", ddlPackBean);
//        DdlPreparedPack.getStringEqualPack(sn, "productSn", ddlPackBean);
        productionRecordRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime","reportStatus", "remark"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue(), errorMsg },ddlPackBean);
    }

    @Override
    public List<MesProductionRecordModel> findMesProductionRecordList(String organizeCode, String productSn) {
        List<MesProductionRecord> productionRecordList = findProductionRecord(organizeCode, productSn);
        List<MesProductionRecordModel> mesProductionRecordModelList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(productionRecordList)) {
            //数据查询
            Map<String, MesCraft> craftMap = getStringMesCraftMap(organizeCode, productionRecordList.stream().map(MesProductionRecord::getCraftCode).distinct().collect(Collectors.toList()));
            Map<String, MesProduceSn> produceSnMap = getStringMesProduceSnMap(organizeCode, productSn);
            Map<String, MesWorkCenter> mesWorkCenterMap = getStringMesWorkCenterMap(organizeCode, productionRecordList.stream().map(MesProductionRecord::getWorkCenterCode).distinct().collect(Collectors.toList()));
            Map<String, MesWorkCell> workCellMap = getStringMesWorkCellMap(organizeCode, productionRecordList.stream().map(MesProductionRecord::getWorkCellCode).distinct().collect(Collectors.toList()));
            //数据组装
            MesProductionRecordModel mesProductionRecordModel = null;
            for (MesProductionRecord mesProductionRecord : productionRecordList) {
                mesProductionRecordModel = new MesProductionRecordModel();
                BeanUtils.copyProperties(mesProductionRecord, mesProductionRecordModel);
                mesProductionRecordModel.setCraftName(Objects.isNull(craftMap) || Objects.isNull(craftMap.get(mesProductionRecord.getCraftCode())) || StringUtils.isEmpty(mesProductionRecord.getCraftCode()) ? "" : craftMap.get(mesProductionRecord.getCraftCode()).getCraftName());
                mesProductionRecordModel.setSnStatus(Objects.isNull(produceSnMap.get(mesProductionRecord.getSerialNumber())) || StringUtils.isEmpty(mesProductionRecord.getSerialNumber()) ? 0 : produceSnMap.get(mesProductionRecord.getSerialNumber()).getSnStatus());
                mesProductionRecordModel.setSnStatusDesc(0 == mesProductionRecordModel.getSnStatus() ?"" : MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(mesProductionRecordModel.getSnStatus()));
                mesProductionRecordModel.setReportStatusDesc(MesExtEnumUtil.REPORT_STATUS.valueOfDescription(mesProductionRecordModel.getReportStatus()));
                mesProductionRecordModel.setWorkCenterName(Objects.isNull(mesWorkCenterMap.get(mesProductionRecord.getWorkCenterCode())) || StringUtils.isEmpty(mesProductionRecord.getWorkCenterCode()) ? "" : mesWorkCenterMap.get(mesProductionRecord.getWorkCenterCode()).getWorkCenterName());
                mesProductionRecordModel.setWorkCellName(Objects.isNull(workCellMap.get(mesProductionRecord.getWorkCellCode())) || StringUtils.isEmpty(mesProductionRecord.getWorkCellCode()) ? "" : workCellMap.get(mesProductionRecord.getWorkCellCode()).getWorkCellName());
                mesProductionRecordModelList.add(mesProductionRecordModel);
            }
        }
        return mesProductionRecordModelList;
    }

    @Override
    public void updateProductionRecord(List<MesProductionRecord> recordList, String userName, Integer reportStatus, String msg) {
        recordList.forEach(record->{
            record.setReportStatus(reportStatus);
            record.setRemark(msg);
            record.setSystemSyncStatus(CommonEnumUtil.FALSE);
            record.setSystemSyncDatetime(MesPcnExtConstWords.EMPTY);
            ConvertBean.serviceModelUpdate(record,userName);
        });
        productionRecordRepository.saveAll(recordList);
    }

    private Map<String, MesCraft> getStringMesCraftMap(String organizeCode, List<String> craftCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(craftCodeList, MesPcnExtConstWords.CRAFT_CODE,packBean);
        List<MesCraft> craftList = mesCraftRepository.findByHqlWhere(packBean);
        Map<String, MesCraft> craftMap = null;
        if(!CollectionUtils.isEmpty(craftList)){
            craftMap = craftList.stream().collect(Collectors.toMap(MesCraft::getCraftCode, t -> t));
        }
        return craftMap;
    }

    private Map<String, MesProduceSn> getStringMesProduceSnMap(String organizeCode, String productSn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productSn, MesPcnExtConstWords.PRODUCT_SN,packBean);
        List<MesProduceSn> mesProduceSns = mesProduceSnRepository.findByHqlWhere(packBean);
        Map<String, MesProduceSn> mesProduceSnMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(mesProduceSns)){
            for (MesProduceSn mesProduceSn : mesProduceSns) {
                if(!mesProduceSnMap.containsKey(mesProduceSn.getSerialNumber())){
                    mesProduceSnMap.put(mesProduceSn.getSerialNumber(),mesProduceSn);
                }
            }
        }
        return mesProduceSnMap;
    }


    private Map<String, MesWorkCenter> getStringMesWorkCenterMap(String organizeCode, List<String> workCenterNoList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(workCenterNoList, MesPcnExtConstWords.WORK_CENTER_CODE,packBean);
        List<MesWorkCenter> mesWorkCenterList = mesWorkCenterRepository.findByHqlWhere(packBean);
        Map<String, MesWorkCenter> mesWorkCenterMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(mesWorkCenterList)){
            for (MesWorkCenter workCenter : mesWorkCenterList) {
                if(!mesWorkCenterMap.containsKey(workCenter.getWorkCenterCode())){
                    mesWorkCenterMap.put(workCenter.getWorkCenterCode(),workCenter);
                }
            }
        }
        return mesWorkCenterMap;
    }

    private Map<String, MesWorkCell> getStringMesWorkCellMap(String organizeCode, List<String> workCenterNoList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(workCenterNoList, MesPcnExtConstWords.WORK_CELL_CODE,packBean);
        List<MesWorkCell> mesWorkCellList = mesWorkCellRepository.findByHqlWhere(packBean);
        Map<String, MesWorkCell> mesProduceSnMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(mesWorkCellList)){
            for (MesWorkCell mesWorkCell : mesWorkCellList) {
                if(!mesProduceSnMap.containsKey(mesWorkCell.getWorkCellCode())){
                    mesProduceSnMap.put(mesWorkCell.getWorkCellCode(),mesWorkCell);
                }
            }
        }
        return mesProduceSnMap;
    }

    public List<MesProductionRecord> findProductionRecord(String organizeCode, String productSn) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(productSn)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productSn,MesPcnExtConstWords.PRODUCT_SN,packBean);
        DdlPreparedPack.getOrderBy(MesPcnExtConstWords.CREATE_DATE_TIME,CommonEnumUtil.ASC_OR_DESC.DESC.getValue(),packBean);
        return productionRecordRepository.findByHqlWhere(packBean);
    }
}
